@@ -1,4 +1,5 @@
 #' Summary of a \code{hlme}, \code{lcmm}, \code{Jointlcmm}, \code{multlcmm},
+#' \code{mpjlcmm}, \code{externSurv}, \code{externX},
 #' \code{epoce} or \code{Diffepoce} objects
 #' 
 #' The function provides a summary of \code{hlme}, \code{lcmm}, \code{multlcmm}
@@ -7,11 +8,12 @@
 #' 
 #' 
 #' @aliases summary.hlme summary.lcmm summary.Jointlcmm summary.multlcmm
-#' summary.epoce summary.Diffepoce
+#' summary.epoce summary.Diffepoce summary.mpjlcmm summary.externSurv summary.externX
 #' @param object an object inheriting from classes \code{hlme}, \code{lcmm},
 #' \code{multlcmm} for fitted latent class mixed-effects, or class
-#' \code{Jointlcmm} for a Joint latent class mixed model or \code{epoce} or
-#' \code{Diffepoce} for predictive accuracy computations.
+#' \code{Jointlcmm}, \code{mpjlcmm} for a Joint latent class mixed model or \code{epoce} or
+#' \code{Diffepoce} for predictive accuracy computations or \code{externSurv}, \code{externX}
+#' for secondary regression models.
 #' @param \dots further arguments to be passed to or from other methods.  They
 #' are ignored in this function.
 #' @return For \code{epoce} or \code{Diffepoce} objects, returns NULL. For
@@ -30,10 +32,14 @@ summary.lcmm <- function(object,...)
 {
     x <- object
     if (!inherits(x, "lcmm")) stop("use only with \"lcmm\" objects")
-
-    cat("General latent class mixed model", "\n")
+    
+    if(inherits(x, "externVar")){
+      cat("Secondary linear mixed model", "\n")
+    } else {
+      cat("General latent class mixed model", "\n")
+    }
     cat("     fitted by maximum likelihood method", "\n")
-
+    
     cl <- x$call
     cl$B <- NULL
     if(is.data.frame(cl$data))
@@ -90,10 +96,21 @@ summary.lcmm <- function(object,...)
     else{
 
         cat(" \n")
-        cat("     Number of iterations: ", x$niter, "\n")
-        cat("     Convergence criteria: parameters=", signif(x$gconv[1],2), "\n")
-        cat("                         : likelihood=", signif(x$gconv[2],2), "\n") 
-        cat("                         : second derivatives=", signif(x$gconv[3],2), "\n")
+        if(inherits(x, "externVar")) {
+          if(x$varest == "paramBoot"){
+            cat("     Proportion of convergence on bootstrap iterations (%)=", x$Mconv, "\n")
+          } else {
+            cat("     Number of iterations: ", x$niter, "\n")
+            cat("     Convergence criteria: parameters=", signif(x$gconv[1],2), "\n")
+            cat("                         : likelihood=", signif(x$gconv[2],2), "\n")
+            cat("                         : second derivatives=", signif(x$gconv[3],2), "\n")
+          }
+        } else {
+          cat("     Number of iterations: ", x$niter, "\n")
+          cat("     Convergence criteria: parameters=", signif(x$gconv[1],2), "\n")
+          cat("                         : likelihood=", signif(x$gconv[2],2), "\n")
+          cat("                         : second derivatives=", signif(x$gconv[3],2), "\n")
+        }
         cat(" \n")
         cat("Goodness-of-fit statistics:", "\n")
         cat(paste("     maximum log-likelihood:", round(x$loglik,2))," \n")
@@ -395,6 +412,7 @@ summary.lcmm <- function(object,...)
                            paste(paste(rep(" ",max(maxch[2]-2,0)),collapse=""),"Se",sep=""),
                            paste(paste(rep(" ",max(maxch[3]-4,0)),collapse=""),"Wald",sep=""),
                            paste(paste(rep(" ",max(maxch[4]-7,0)),collapse=""),"p-value",sep=""))
+        if(inherits(x, "externVar")) colnames(tmp)[2] = paste(paste(rep(" ",max(maxch[2]-4,0)),collapse=""),"Se**",sep="")
         cat("\n")
         print(tmp,quote=FALSE,na.print="")
         cat("\n")
@@ -403,7 +421,12 @@ summary.lcmm <- function(object,...)
             {
                 cat(" * coefficient fixed by the user \n \n")
             }
-
+        if(inherits(x, "externVar")){
+          if(x$varest == "none") cat(" ** total variance estimated witout correction for primary model uncertainty", "\n \n")
+          if(x$varest == "Hessian") cat(" ** total variance estimated through the Hessian of the joint likelihood", "\n \n")
+          if(x$varest == "paramBoot") cat(" ** total variance estimated through parametric bootstrap", "\n \n")
+        }
+        
         return(invisible(tTable))
     }
 }
